"""Gauss--Lobatto--Legendre quadrature rule."""

import numpy as np
import numpy.typing as npt
from quadraturerules.domain import Domain
import typing


def gauss_lobatto_legendre(
    domain: Domain,
    order: int,
) -> typing.Tuple[npt.NDArray[np.float64], npt.NDArray[np.float64]]:
    """Get a Gauss--Lobatto--Legendre quadrature rule."""
    match domain:
        case Domain.SquareBasedPyramid:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Triangle:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Quadrilateral:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Hexahedron:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Interval:
            match order:
                case 0:
                    return np.array([[1.0, 0.0], [0.0, 1.0]]), np.array(
                        [0.4999999999999999, 0.4999999999999999]
                    )
                case 1:
                    return np.array([[1.0, 0.0], [0.5, 0.5], [0.0, 1.0]]), np.array(
                        [0.1666666666666669, 0.6666666666666665, 0.16666666666666685]
                    )
                case 2:
                    return np.array(
                        [
                            [1.0, 0.0],
                            [0.7236067977499789, 0.276393202250021],
                            [0.27639320225002106, 0.7236067977499789],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.08333333333333319,
                            0.4166666666666669,
                            0.41666666666666624,
                            0.08333333333333333,
                        ]
                    )
                case 3:
                    return np.array(
                        [
                            [0.9999999999999999, 1.1102230246251565e-16],
                            [0.8273268353539887, 0.1726731646460114],
                            [0.5, 0.49999999999999994],
                            [0.17267316464601157, 0.8273268353539884],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.05000000000000011,
                            0.27222222222222214,
                            0.35555555555555557,
                            0.2722222222222227,
                            0.04999999999999995,
                        ]
                    )
                case 4:
                    return np.array(
                        [
                            [0.9999999999999998, 2.7755575615628914e-16],
                            [0.8825276619647323, 0.11747233803526769],
                            [0.6426157582403226, 0.3573842417596774],
                            [0.35738424175967753, 0.6426157582403225],
                            [0.11747233803526769, 0.8825276619647323],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.03333333333333342,
                            0.18923747814892322,
                            0.27742918851774345,
                            0.2774291885177426,
                            0.1892374781489239,
                            0.03333333333333339,
                        ]
                    )
                case 5:
                    return np.array(
                        [
                            [1.0000000000000002, -2.220446049250313e-16],
                            [0.9151119481392833, 0.08488805186071663],
                            [0.7344243967353571, 0.2655756032646429],
                            [0.4999999999999998, 0.5000000000000002],
                            [0.265575603264643, 0.734424396735357],
                            [0.08488805186071657, 0.9151119481392834],
                            [-2.220446049250313e-16, 1.0000000000000002],
                        ]
                    ), np.array(
                        [
                            0.02380952380952369,
                            0.13841302368078276,
                            0.21587269060493128,
                            0.24380952380952378,
                            0.2158726906049315,
                            0.1384130236807831,
                            0.023809523809523666,
                        ]
                    )
                case 6:
                    return np.array(
                        [
                            [1.0000000000000002, -2.220446049250313e-16],
                            [0.9358700742548034, 0.06412992574519655],
                            [0.7958500907165712, 0.2041499092834288],
                            [0.6046496089512394, 0.3953503910487606],
                            [0.39535039104876035, 0.6046496089512396],
                            [0.20414990928342858, 0.7958500907165714],
                            [0.0641299257451966, 0.9358700742548034],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.017857142857142815,
                            0.10535211357175311,
                            0.17056134624175223,
                            0.20622939732935186,
                            0.20622939732935172,
                            0.17056134624175218,
                            0.10535211357175316,
                            0.01785714285714291,
                        ]
                    )
                case 7:
                    return np.array(
                        [
                            [1.0, 0.0],
                            [0.9498789977057301, 0.050121002294269856],
                            [0.8385931397553688, 0.16140686024463125],
                            [0.6815587319130891, 0.3184412680869109],
                            [0.5, 0.49999999999999994],
                            [0.318441268086911, 0.681558731913089],
                            [0.16140686024463102, 0.838593139755369],
                            [0.050121002294269856, 0.9498789977057301],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.013888888888888826,
                            0.08274768078040293,
                            0.13726935625008083,
                            0.1732142554865229,
                            0.18575963718820857,
                            0.17321425548652275,
                            0.13726935625008096,
                            0.08274768078040268,
                            0.013888888888888928,
                        ]
                    )
                case 8:
                    return np.array(
                        [
                            [1.0000000000000002, -2.220446049250313e-16],
                            [0.9597669540832292, 0.04023304591677079],
                            [0.8693869325527526, 0.1306130674472475],
                            [0.7389624749052224, 0.2610375250947777],
                            [0.5826394788331934, 0.4173605211668066],
                            [0.41736052116680644, 0.5826394788331936],
                            [0.26103752509477784, 0.7389624749052222],
                            [0.13061306744724743, 0.8693869325527526],
                            [0.04023304591677068, 0.9597669540832293],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.011111111111110999,
                            0.06665299542553525,
                            0.11244467103156307,
                            0.1460213418398421,
                            0.1637698805919486,
                            0.16376988059194894,
                            0.1460213418398421,
                            0.11244467103156307,
                            0.06665299542553495,
                            0.011111111111110942,
                        ]
                    )
                case 9:
                    return np.array(
                        [
                            [1.0000000000000002, -2.220446049250313e-16],
                            [0.9670007152040294, 0.032999284795970585],
                            [0.892241736831572, 0.1077582631684279],
                            [0.7826176634981022, 0.21738233650189775],
                            [0.6478790677934695, 0.3521209322065305],
                            [0.5, 0.49999999999999994],
                            [0.3521209322065304, 0.6478790677934696],
                            [0.21738233650189764, 0.7826176634981024],
                            [0.10775826316842785, 0.8922417368315722],
                            [0.032999284795970585, 0.9670007152040294],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.009090909090909042,
                            0.05480613663349759,
                            0.0935849408901526,
                            0.12402405213201415,
                            0.14343956238950414,
                            0.15010879772784577,
                            0.14343956238950406,
                            0.12402405213201481,
                            0.09358494089015215,
                            0.05480613663349694,
                            0.009090909090908957,
                        ]
                    )
                case 10:
                    return np.array(
                        [
                            [1.0, 5.551115123125783e-17],
                            [0.972449636111441, 0.02755036388855897],
                            [0.9096396608220033, 0.09036033917799668],
                            [0.8164380765159305, 0.18356192348406952],
                            [0.6997654704826746, 0.30023452951732543],
                            [0.5682764664274638, 0.43172353357253623],
                            [0.43172353357253634, 0.5682764664274637],
                            [0.30023452951732554, 0.6997654704826745],
                            [0.18356192348406974, 0.8164380765159303],
                            [0.09036033917799635, 0.9096396608220036],
                            [0.027550363888559026, 0.972449636111441],
                            [2.220446049250313e-16, 0.9999999999999998],
                        ]
                    ), np.array(
                        [
                            0.007575757575757551,
                            0.0458422587065979,
                            0.07898735278218531,
                            0.10625420888051049,
                            0.12563780159960067,
                            0.13570262045534826,
                            0.13570262045534784,
                            0.1256378015996004,
                            0.10625420888051092,
                            0.07898735278218491,
                            0.0458422587065979,
                            0.0075757575757577555,
                        ]
                    )
                case 11:
                    return np.array(
                        [
                            [1.0, -1.1102230246251565e-16],
                            [0.9766549233210818, 0.023345076678918164],
                            [0.9231737823259362, 0.07682621767406383],
                            [0.8430942345408787, 0.15690576545912133],
                            [0.7414549105456681, 0.25854508945433186],
                            [0.6246434650531201, 0.37535653494687987],
                            [0.5, 0.5],
                            [0.3753565349468799, 0.6246434650531201],
                            [0.25854508945433197, 0.741454910545668],
                            [0.15690576545912105, 0.843094234540879],
                            [0.07682621767406395, 0.923173782325936],
                            [0.023345076678918053, 0.976654923321082],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.006410256410256359,
                            0.038900843373409474,
                            0.06749096334480395,
                            0.09182343260177506,
                            0.11038389678305505,
                            0.12200789515333801,
                            0.12596542466672317,
                            0.12200789515333801,
                            0.11038389678305545,
                            0.09182343260177549,
                            0.06749096334480419,
                            0.038900843373409336,
                            0.006410256410256437,
                        ]
                    )
                case 12:
                    return np.array(
                        [
                            [0.9999999999999999, 1.1102230246251565e-16],
                            [0.9799675226336305, 0.020032477366369483],
                            [0.9339005269151737, 0.06609947308482633],
                            [0.8644342995456633, 0.13556570045433675],
                            [0.7753197014643236, 0.22468029853567634],
                            [0.6713620066713566, 0.3286379933286434],
                            [0.5581659344418517, 0.4418340655581483],
                            [0.44183406555814786, 0.5581659344418521],
                            [0.3286379933286435, 0.6713620066713565],
                            [0.22468029853567628, 0.7753197014643237],
                            [0.1355657004543369, 0.8644342995456631],
                            [0.06609947308482633, 0.9339005269151737],
                            [0.020032477366369594, 0.9799675226336304],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.005494505494505587,
                            0.0334186422488406,
                            0.05829332794935556,
                            0.08001092588147618,
                            0.09741307468670797,
                            0.10956312650488567,
                            0.11580639723422881,
                            0.11580639723422878,
                            0.10956312650488538,
                            0.09741307468670836,
                            0.08001092588147612,
                            0.05829332794935564,
                            0.033418642248840794,
                            0.005494505494505398,
                        ]
                    )
                case 13:
                    return np.array(
                        [
                            [1.0, 0.0],
                            [0.9826229632519193, 0.017377036748080665],
                            [0.9425410221114883, 0.057458977888511786],
                            [0.8817598449759079, 0.11824015502409219],
                            [0.8031266027349229, 0.1968733972650772],
                            [0.7103190273568364, 0.2896809726431636],
                            [0.6076769776818971, 0.3923230223181029],
                            [0.5, 0.5],
                            [0.3923230223181029, 0.6076769776818971],
                            [0.2896809726431637, 0.7103190273568363],
                            [0.19687339726507713, 0.8031266027349229],
                            [0.11824015502409235, 0.8817598449759076],
                            [0.05745897788851173, 0.9425410221114883],
                            [0.017377036748080554, 0.9826229632519194],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.004761904761904712,
                            0.029014946514300585,
                            0.050830035162859034,
                            0.07025584990121428,
                            0.08639482362680054,
                            0.09849361798230638,
                            0.10598679296341056,
                            0.10852405817440798,
                            0.1059867929634102,
                            0.09849361798230694,
                            0.08639482362680041,
                            0.07025584990121425,
                            0.05083003516285919,
                            0.0290149465143005,
                            0.004761904761904956,
                        ]
                    )
                case 14:
                    return np.array(
                        [
                            [1.0, -1.1102230246251565e-16],
                            [0.9847840231351088, 0.015215976864891123],
                            [0.9496002665467362, 0.05039973345326382],
                            [0.8960041459309077, 0.10399585406909229],
                            [0.8261943514412465, 0.17380564855875347],
                            [0.7430297109435687, 0.25697028905643127],
                            [0.6499152344503816, 0.35008476554961837],
                            [0.5506631367609747, 0.44933686323902533],
                            [0.4493368632390252, 0.5506631367609748],
                            [0.3500847655496182, 0.6499152344503818],
                            [0.2569702890564314, 0.7430297109435686],
                            [0.17380564855875336, 0.8261943514412466],
                            [0.10399585406909262, 0.8960041459309074],
                            [0.050399733453263984, 0.949600266546736],
                            [0.01521597686489129, 0.9847840231351087],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.004166666666666632,
                            0.025425180502960033,
                            0.044696848662965215,
                            0.06212769106625702,
                            0.07701349040358248,
                            0.08874595669585181,
                            0.09684501191260195,
                            0.1009791540891151,
                            0.1009791540891149,
                            0.09684501191260142,
                            0.0887459566958526,
                            0.07701349040358242,
                            0.06212769106625717,
                            0.044696848662965215,
                            0.02542518050295991,
                            0.0041666666666664845,
                        ]
                    )
                case 15:
                    return np.array(
                        [
                            [0.9999999999999998, 2.7755575615628914e-16],
                            [0.9865660883157092, 0.0134339116842907],
                            [0.9554399979577869, 0.04456000204221311],
                            [0.907848125610885, 0.09215187438911493],
                            [0.8455144903138425, 0.15448550968615754],
                            [0.7706926996650507, 0.22930730033494923],
                            [0.6860872167827388, 0.3139127832172613],
                            [0.5947559867591588, 0.40524401324084125],
                            [0.5000000000000002, 0.49999999999999983],
                            [0.4052440132408416, 0.5947559867591584],
                            [0.31391278321726157, 0.6860872167827384],
                            [0.22930730033494906, 0.7706926996650509],
                            [0.15448550968615749, 0.8455144903138425],
                            [0.09215187438911476, 0.9078481256108852],
                            [0.044560002042213, 0.955439997957787],
                            [0.013433911684290534, 0.9865660883157095],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.003676470588235441,
                            0.02246097027162666,
                            0.03959913525184375,
                            0.05529645450351427,
                            0.06899387310096329,
                            0.08019733099881056,
                            0.08850212675782908,
                            0.0936081698388097,
                            0.0953309373767344,
                            0.09360816983880912,
                            0.08850212675782915,
                            0.08019733099881093,
                            0.06899387310096344,
                            0.05529645450351387,
                            0.03959913525184312,
                            0.022460970271627308,
                            0.0036764705882355213,
                        ]
                    )
                case 16:
                    return np.array(
                        [
                            [1.0, -1.1102230246251565e-16],
                            [0.9880527787060993, 0.011947221293900634],
                            [0.960324592673767, 0.03967540732623298],
                            [0.9177967676090452, 0.08220323239095484],
                            [0.8618396646416215, 0.13816033535837852],
                            [0.7942524171593308, 0.20574758284066919],
                            [0.7172075184560621, 0.2827924815439379],
                            [0.6331813264391405, 0.3668186735608595],
                            [0.5448745467423262, 0.4551254532576739],
                            [0.4551254532576742, 0.5448745467423258],
                            [0.3668186735608596, 0.6331813264391404],
                            [0.282792481543938, 0.717207518456062],
                            [0.20574758284066896, 0.794252417159331],
                            [0.13816033535837846, 0.8618396646416215],
                            [0.08220323239095473, 0.9177967676090453],
                            [0.03967540732623287, 0.9603245926737671],
                            [0.011947221293900467, 0.9880527787060995],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.003267973856209061,
                            0.01998531440545694,
                            0.03531858344281688,
                            0.04950813585875151,
                            0.06210526656648326,
                            0.07270598078690123,
                            0.08096975861880143,
                            0.08663105474472822,
                            0.08950793171985133,
                            0.08950793171985152,
                            0.08663105474472789,
                            0.08096975861880143,
                            0.07270598078690084,
                            0.06210526656648361,
                            0.04950813585875187,
                            0.035318583442816624,
                            0.019985314405457047,
                            0.003267973856209243,
                        ]
                    )
                case 17:
                    return np.array(
                        [
                            [1.0, -1.1102230246251565e-16],
                            [0.9893058831110401, 0.010694116888959937],
                            [0.9644507640762929, 0.03554923592370701],
                            [0.9262302888983229, 0.07376971110167702],
                            [0.8757471012763064, 0.1242528987236936],
                            [0.8144540686326103, 0.18554593136738978],
                            [0.7441146428403568, 0.2558853571596432],
                            [0.6667524239122492, 0.3332475760877508],
                            [0.584593011704641, 0.4154069882953591],
                            [0.5, 0.49999999999999994],
                            [0.41540698829535916, 0.5845930117046408],
                            [0.3332475760877507, 0.6667524239122493],
                            [0.2558853571596432, 0.7441146428403568],
                            [0.18554593136738973, 0.8144540686326103],
                            [0.1242528987236935, 0.8757471012763065],
                            [0.07376971110167696, 0.926230288898323],
                            [0.03554923592370707, 0.9644507640762929],
                            [0.010694116888959937, 0.9893058831110401],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.0029239766081870823,
                            0.017896682593088103,
                            0.0316909458813149,
                            0.04456587854960388,
                            0.05615767073865237,
                            0.06613364022437511,
                            0.07420697129796952,
                            0.08014546202203048,
                            0.0837782922635714,
                            0.0850009596424136,
                            0.08377829226357138,
                            0.0801454620220308,
                            0.07420697129796958,
                            0.06613364022437511,
                            0.05615767073865217,
                            0.044565878549603415,
                            0.0316909458813149,
                            0.017896682593088298,
                            0.0029239766081868954,
                        ]
                    )
                case 18:
                    return np.array(
                        [
                            [0.9999999999999999, 1.1102230246251565e-16],
                            [0.9903718524469571, 0.009628147553042865],
                            [0.9679672494063327, 0.03203275059366728],
                            [0.933438989044975, 0.06656101095502492],
                            [0.8876841304760279, 0.11231586952397205],
                            [0.8318882011451557, 0.16811179885484429],
                            [0.7674964320159432, 0.2325035679840568],
                            [0.6961765918569547, 0.3038234081430453],
                            [0.619775852961493, 0.380224147038507],
                            [0.5402729686194108, 0.4597270313805892],
                            [0.4597270313805891, 0.5402729686194109],
                            [0.3802241470385067, 0.6197758529614933],
                            [0.3038234081430452, 0.6961765918569548],
                            [0.2325035679840568, 0.7674964320159432],
                            [0.1681117988548444, 0.8318882011451556],
                            [0.11231586952397232, 0.8876841304760277],
                            [0.06656101095502498, 0.933438989044975],
                            [0.03203275059366728, 0.9679672494063327],
                            [0.009628147553043087, 0.9903718524469569],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.002631578947368505,
                            0.016118561594244192,
                            0.028590901063783602,
                            0.040315881998059584,
                            0.05099574984972538,
                            0.06035461381433761,
                            0.06815024117936189,
                            0.0741807770354587,
                            0.07829005132373774,
                            0.08037164319392277,
                            0.08037164319392273,
                            0.07829005132373797,
                            0.07418077703545829,
                            0.06815024117936165,
                            0.060354613814337724,
                            0.05099574984972575,
                            0.040315881998059584,
                            0.028590901063783338,
                            0.0161185615942447,
                            0.002631578947368365,
                        ]
                    )
                case 19:
                    return np.array(
                        [
                            [1.0000000000000002, -2.220446049250313e-16],
                            [0.9912861483022739, 0.008713851697726038],
                            [0.9709881484798727, 0.029011851520127308],
                            [0.9396473776617952, 0.06035262233820482],
                            [0.8980009630388561, 0.10199903696114393],
                            [0.8470255130311115, 0.1529744869688885],
                            [0.7879159801309153, 0.21208401986908465],
                            [0.722057891639501, 0.27794210836049904],
                            [0.6509949282543823, 0.34900507174561773],
                            [0.5763927579010925, 0.42360724209890754],
                            [0.49999999999999967, 0.5000000000000003],
                            [0.4236072420989072, 0.5763927579010928],
                            [0.34900507174561757, 0.6509949282543824],
                            [0.27794210836049915, 0.7220578916395008],
                            [0.21208401986908443, 0.7879159801309156],
                            [0.1529744869688885, 0.8470255130311115],
                            [0.10199903696114399, 0.898000963038856],
                            [0.06035262233820493, 0.9396473776617951],
                            [0.02901185152012742, 0.9709881484798726],
                            [0.008713851697726205, 0.9912861483022738],
                            [1.1102230246251565e-16, 0.9999999999999999],
                        ]
                    ), np.array(
                        [
                            0.0023809523809523686,
                            0.014592420049252883,
                            0.025921584500424606,
                            0.03663695909253699,
                            0.04649273397894329,
                            0.05525854160956152,
                            0.06272906059543445,
                            0.06872923143002083,
                            0.0731184312239889,
                            0.07579378755584021,
                            0.07669259516608694,
                            0.07579378755584082,
                            0.07311843122398853,
                            0.06872923143002124,
                            0.06272906059543448,
                            0.05525854160956174,
                            0.04649273397894334,
                            0.03663695909253697,
                            0.025921584500424692,
                            0.014592420049252765,
                            0.0023809523809522693,
                        ]
                    )
                case 20:
                    return np.array(
                        [
                            [0.9999999999999996, 4.996003610813204e-16],
                            [0.992076219228823, 0.007923780771177003],
                            [0.9736021419996146, 0.026397858000385466],
                            [0.9450311450954522, 0.05496885490454784],
                            [0.9069744638059606, 0.09302553619403942],
                            [0.8602436199806007, 0.13975638001939922],
                            [0.8058347191421293, 0.19416528085787071],
                            [0.7449074375949509, 0.25509256240504913],
                            [0.6787603550694598, 0.32123964493054025],
                            [0.6088032925796427, 0.3911967074203573],
                            [0.5365272700054491, 0.4634727299945509],
                            [0.4634727299945508, 0.5365272700054492],
                            [0.3911967074203575, 0.6088032925796425],
                            [0.32123964493054025, 0.6787603550694598],
                            [0.25509256240504885, 0.7449074375949511],
                            [0.19416528085787066, 0.8058347191421293],
                            [0.13975638001939905, 0.860243619980601],
                            [0.09302553619403964, 0.9069744638059604],
                            [0.05496885490454784, 0.9450311450954522],
                            [0.026397858000385854, 0.9736021419996141],
                            [0.007923780771177058, 0.9920762192288229],
                            [0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.0021645021645025028,
                            0.013272873841250659,
                            0.023607232646870206,
                            0.033432802932276434,
                            0.042545030195919584,
                            0.05075028740082406,
                            0.05787382232696931,
                            0.06376384832671506,
                            0.06829484430687037,
                            0.07137024613568074,
                            0.07292450972212078,
                            0.07292450972212071,
                            0.07137024613568128,
                            0.06829484430687058,
                            0.0637638483267155,
                            0.05787382232696952,
                            0.0507502874008239,
                            0.04254503019591946,
                            0.03343280293227698,
                            0.02360723264687059,
                            0.013272873841250777,
                            0.0021645021645019845,
                        ]
                    )
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.TriangularPrism:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Tetrahedron:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
